#include <stdio.h>
#include <stdlib.h>

#define F_CPU 16000000

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include "helpers.h"

#define LIGHT_UP_WHOLE 0
#define TOP_DOWN_ANIMATION 1
#define RANDOM_LED 2

static int effectIndex = LIGHT_UP_WHOLE;

#define EFFECT_COUNT 3

unsigned int A[8] = {PA0, PA1, PA2, PA3, PA4, PA5, PA6, PA7};
unsigned int B[8] = {PB0, PB1, PB2, PB3, PB4, PB5, PB6, PB7};

ISR(TIMER1_COMPA_vect)
{
	static bool PB2pressed = false;

	if ((PINB & (1 << PB2)) == 0 && !PB2pressed) {
		effectIndex = (effectIndex + 1) % EFFECT_COUNT;

		PB2pressed = true;
	} else if ((PINB & (1 << PB2)) != 0) {
		PB2pressed = false;
	}
}

void light_up_whole(void)
{
	static int level = LEVEL1;

	PORTA = 0xFF;
	PORTB = 0xFF;

	turn_on_level(level);
	_delay_ms(2);
	turn_off_level(level);

	level = (level == 0) ? 3 : (level - 1);
}

void top_down_animation(void)
{
	static int level = 0;
	static int direction = 1;

	turn_on_level(level);
	_delay_ms(200);
	turn_off_level(level);

	if (level == LEVEL4 && direction == 1) {
		direction = -1;
		_delay_ms(200);
	}
	if (level == LEVEL1 && direction == -1) {
		direction = 1;
	}
	level += direction;
}

void random_led()
{
	turn_on_level(rand() % 4);

	if (rand() % 2)
		PORTA |= (1 << A[rand() % 8]);
	else
		PORTB |= (1 << B[rand() % 8]);
	_delay_ms(100);
}

int main(void)
{
	init();

	while (1) {
		reset();

		if (effectIndex == LIGHT_UP_WHOLE) {
			light_up_whole();
		}
		if (effectIndex == TOP_DOWN_ANIMATION) {
			top_down_animation();
		}
		if (effectIndex == RANDOM_LED) {
			random_led();
		}
	}

	return 0;
}
